package com.neolix.cubingsdk;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.PixelFormat;
import android.graphics.Point;
import android.graphics.PorterDuff;
import android.os.Bundle;
import android.os.Environment;
import android.os.SystemClock;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.TextureView;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.neolix.cubingsdk.utils.MeasureResultData;
import com.neolix.cubingsdk.utils.Util;
import com.tof.tofcubing.BitmapUtil;
import com.tof.tofcubing.CameraApi;
import com.tof.tofcubing.CanvasUtils;
import com.tof.tofcubing.IProcessCallback;
import com.tof.tofcubing.MEASUREMODE;
import com.tof.tofcubing.MeasureState;
import com.tof.tofcubing.TofCubing;
import com.tof.tofcubing.ViewfinderView;
import com.tof.tofcubing.permission.OnPermission;
import com.tof.tofcubing.permission.PermissionUtils;
import com.tof.tofcubing.permission.SimplePermissions;

import java.io.File;
import java.util.List;




public class MainActivity extends Activity implements IProcessCallback {

    private static final String TAG = "zhg_MainActivity";
    private List<String> deniedList;

    //--------------------------------------------------------------
    //测量结果View start
    private LinearLayout resultLayout;
    private TextView retWidth, retLength, retHeight, retVolume;
    private Button btReMeasure, btSubmit;
    //测量结果View end
    //--------------------------------------------------------------


    //测量按钮
    private ImageButton measureBnt;
    private CheckBox bigObjectCheck;
    private ViewfinderView viewfinderView;

    //主页面View end
    //--------------------------------------------------------------


    volatile boolean measureFinish = false;//测量结束
    private float length = .0f, width = .0f, height = .0f, volume = 0.0f;
    private float bigLen1 = .0f, bigWid1 = .0f;

    private TofCubing tofCubing;
    private MeasureState measureState;
    public TextureView mainTextureView;
    public volatile boolean isMeasureOkView = false;
    private SurfaceView outlineSurfaceView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_sfcubing);
        Log.i(TAG, "onCreate...");
        createTofCubingSDK(this);
        initPaint();
    }


    @Override
    protected void onResume() {
        super.onResume();
        initView();
        //保持屏幕常亮
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        Log.i(TAG, "onResume init");
        checkPermission();
        tofCubing.onResuemTofCubing();

        outlineSurfaceView.setZOrderOnTop(true);
        outlineSurfaceView.getHolder().setFormat(PixelFormat.TRANSLUCENT);
        //设置测量模式.可更改测量模式
        measureState.setMode(MEASUREMODE.Cube);
    }

    private void createTofCubingSDK(Activity activity) {
        if (tofCubing == null) {
            tofCubing = new TofCubing(activity, this, getMainTextureView());
        }
        tofCubing.softPauseCamera(TofCubing.CAMERA_PAUSE_NOTHING);
        measureState = MeasureState.getMesaureStateInstance(tofCubing);
    }

    @Override
    protected void onPause() {
        super.onPause();
        getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        Log.i(TAG, "onPause");
        if (tofCubing != null) {
            tofCubing.onPauseTofCubing();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        Log.i(TAG, "onDestroy");
        if (tofCubing != null) {
            tofCubing.onDestoryTofCubing();
        }
        measureState = null;
        tofCubing = null;
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
        finish();
    }

    @Override
    public void processCallback(byte[] rgbsync, float[] bounding, int measureMode, int length, int width, int height, float dist, float angle, float tofTemp) {
        updateResult(measureMode, length, width, height, dist, angle, tofTemp);
        updatePointData(bounding);
        updateOutLine();
        updateMeasureCompleteView(rgbsync);
    }

    @Override
    public void onFaceBackCameraData(byte[] bytes) {

    }

    @Override
    public void onError(int i, String s) {

    }

    @Override
    public void onAverageResultCallback(int i, int i1, int i2) {

    }

    private Point[] cubePoints = new Point[8];
    /**
     * 更新物体坐标到cubePoints
     * */
    private void updatePointData(float[] bounding) {
        int j = 0;
        for (int i = 0; i < cubePoints.length; i++) {
            if (cubePoints[i] == null) {
                cubePoints[i] = new Point(0, 0);
            }
            cubePoints[i].x = (int) bounding[j++];
            cubePoints[i].y = (int) bounding[j++];
        }
    }
    /**
     * 重新绘制物体坐标
     */
    private void updateOutLine(){
        if (tofCubing.getCameraMode() == TofCubing.CAMERA_PAUSE_NOTHING) {
            //根据cubePoints将坐标绘制到tofSurfaceView
            SurfaceHolder holder = outlineSurfaceView.getHolder();
            Canvas canvas = holder.lockCanvas();
            if (canvas == null) return;
            canvas.rotate(90);
            canvas.translate(0, -CameraApi.PREVIEW_HEIGHT);
            canvas.drawColor(Color.TRANSPARENT, PorterDuff.Mode.CLEAR);
            CanvasUtils.drawOutLine(canvas, cubePoints, 14, 18, false);
            holder.unlockCanvasAndPost(canvas);
            isMeasureOkView = false;
        }
    }

    /**
     * @param rgbsync
     */
    private void updateMeasureCompleteView(byte[] rgbsync) {
        Log.e(TAG, " updateMeasureCompleteView CameraMode=" + tofCubing.getCameraMode());
        if (tofCubing.getCameraMode() != TofCubing.CAMERA_PAUSE_NOTHING) {
            Log.e(TAG, "updateMeasureCompleteView 1in");
            drawCameraBitmap(rgbsync);
            if (measureState.getCurrentMode() == MEASUREMODE.Large || measureState.getCurrentMode() == MEASUREMODE.LargeIrreg) {
                //调整结果更新，大物体第一面
                /*if (!measureState.isInMeasureLargeObject()) {
                    //调整结果更新，大物体第一面
                    bigLen1 = Util.getFormatNumber(callbackMeasureResult.getLength(), 1);
                    bigWid1 = Util.getFormatNumber(callbackMeasureResult.getWidth(), 1);
                    handler.sendEmptyMessage(SHOW_BIGMODE_FIRST_SIDEVIEW);
                } else {
                    //调整结果更新，大物体第二面
                    handler.sendEmptyMessage(SHOW_BIGMODE_SECOND_SIDEVIEW);
                }*/
            }
        }
    }

    private ImageView mCameraImage;
    /**
     * Activity的SurfaceView中， 显示测量完成的图片
     *
     * @param rgbsrc
     */
    private void drawCameraBitmap(byte[] rgbsrc) {
        Log.i(TAG, "drawCameraBitmap: ");
        Bitmap bitmap = drawCameraBitmap2(rgbsrc);
        if (callbackMeasureResult.getLength() > 0) {
            showCameraImageView(bitmap);
        }
    }

    byte[] rgbSync;
    Paint[] linePaints = new Paint[4], pointPaints = new Paint[4];

    public Bitmap drawCameraBitmap2(byte[] rgbsrc) {
        Log.i(TAG, "drawCameraBitmap2: ");
        isMeasureOkView = true;
        rgbSync = rgbsrc;
        Bitmap rgbBitmap = BitmapUtil.nv21ToBitmap(this, rgbsrc, CameraApi.PREVIEW_WIDTH, CameraApi.PREVIEW_HEIGHT);
        if (cubePoints != null) {
            if (rgbBitmap != null && rgbBitmap.isMutable()) {
                Canvas canvasbitmap = new Canvas(rgbBitmap);
                Point[] pointsTmp = new Point[cubePoints.length];
                System.arraycopy(cubePoints, 0, pointsTmp, 0, cubePoints.length);
                reverseAdjustmentCalibratePoint(pointsTmp);
                if (cubePoints[4].x != 0 && cubePoints[6].x != 0) {
                    BitmapUtil.drawcube(rgbBitmap, cubePoints, 14, 18);
                } else {
                    CanvasUtils.drawQuadrilateral(canvasbitmap, cubePoints, pointPaints, linePaints, false);
                }
            }
        }
        rgbBitmap = BitmapUtil.RotateBitmap(90, rgbBitmap);
        return rgbBitmap;
    }

    /**
     * 调整校准坐标,根据相机屏幕宽高和canvas和SurfaceView大小
     */
    private void reverseAdjustmentCalibratePoint(Point[] points) {
        Log.i("zouhuagang", "adjustmentCalibratePoint: ");
        for (int i = 0; i < points.length; i++) {
            if (points[i] == null) {
                continue;
            }
            float ratio = 1.0f * 1280 / CameraApi.PREVIEW_WIDTH;
            points[i].x = (int) ((points[i].x - 1280 / 2) / ratio + CameraApi.PREVIEW_WIDTH / 2);
            points[i].y = (int) ((points[i].y - 720) / ratio + CameraApi.PREVIEW_HEIGHT / 2);
        }
    }

    //点击生成测量结果
    public void measure() {
        Log.i(TAG, "measure: ");
        tofCubing.softPauseCamera(TofCubing.CAMERA_PAUSE_ALL);
        SystemClock.sleep(200);
        //没有测量数据 返回，  callbackMeasureResult记录的数据
        if (callbackMeasureResult.getLength() <= 0 || !isMeasureOkView) {
            tofCubing.softPauseCamera(TofCubing.CAMERA_PAUSE_NOTHING);
            disMissCameraImageView();
            return;
        }
        //确保有值，保存
        //......
        //更新测量结果到activity
        showMeasureResutView();
    }

    public void reMeasure() {
        //1. 清空大物体第一次数据
        //2. 更新状态
        //3. 恢复相机
        disMissCameraImageView();
        cleanMeasureResutView();
        tofCubing.softPauseCamera(TofCubing.CAMERA_PAUSE_NOTHING);

    }

    public void showCameraImageView(final Bitmap rgbBitmap) {
        Log.i(TAG, "showCameraImageView: ");
        this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                outlineSurfaceView.setVisibility(View.GONE);
                mCameraImage.setVisibility(View.VISIBLE);
                mCameraImage.setImageBitmap(rgbBitmap);
            }
        });
    }

    public void disMissCameraImageView() {
        Log.i(TAG, "disMissCameraImageView: ");
        this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                mCameraImage.setVisibility(View.GONE);
                outlineSurfaceView.setVisibility(View.VISIBLE);
            }
        });
    }

    private MeasureResultData callbackMeasureResult = new MeasureResultData();
    private MeasureResultData measureCompletedResult = new MeasureResultData();
    /**
     * 更新测量结果到 测量结果
     *
     * @param length
     * @param width
     * @param height
     */
    private void updateResult(int measureMode, int length, int width, int height, float dist, float angle, float tofTemp) {
        //callbackMeasureResult记录结果，用于展示到用户界面
        callbackMeasureResult.setLength(length / 10f);
        callbackMeasureResult.setWidth(width / 10f);
        callbackMeasureResult.setHeight(height / 10f);
        callbackMeasureResult.setMeasureMode(measureMode);
        callbackMeasureResult.setDistToObject(dist);
        callbackMeasureResult.setAngle(angle);
        callbackMeasureResult.setTofTemperature(tofTemp);
    }

    private void initView() {
        //初始化测量界面
        initMeasureReadyView();
        //初始化测量结果界面
        initMeasureResultView();
        //展示开始测量界面
        showMeasureReadyViewAndInit();
    }

    public TextureView getMainTextureView() {
        if (mainTextureView == null) {
            mainTextureView = findViewById(R.id.maincamera_surface_view);
        }
        return mainTextureView;
    }

    //初始化-测量开始界面
    private void initMeasureReadyView() {
        mainTextureView = findViewById(R.id.maincamera_surface_view);
        outlineSurfaceView = findViewById(R.id.outline_surface_view);
        mCameraImage = findViewById(R.id.camera_image);
        viewfinderView = (ViewfinderView) findViewById(R.id.viewfinder_view);
        measureBnt = (ImageButton) findViewById(R.id.btn_measure);
        bigObjectCheck = (CheckBox) findViewById(R.id.bigobject_check);
        bigObjectCheck.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                if(bigObjectCheck.isChecked()){
                    measureState.setMode(MEASUREMODE.Large);
                }else{
                    measureState.setMode(MEASUREMODE.Cube);
                }
            }
        });
    }

    //初始化-测量完成界面
    private void initMeasureResultView() {
        //长宽高整体布局
        resultLayout = (LinearLayout) findViewById(R.id.measure_result_layout);
        //长宽高体积
        retWidth = (TextView) findViewById(R.id.width_ret);
        retLength = (TextView) findViewById(R.id.length_ret);
        retHeight = (TextView) findViewById(R.id.height_ret);
        retVolume = (TextView) findViewById(R.id.volume_ret);
        //重新测量提交
        btReMeasure = (Button) findViewById(R.id.bt_remeasure);
        btSubmit = (Button) findViewById(R.id.bt_submit);
    }

    //显示开始测量界面，不包含业务逻辑仅界面
    private void setMeasureReadyViewVisibility(int state) {
        measureBnt.setVisibility(state);
        bigObjectCheck.setVisibility(state);
    }

    //显示测量结果界面，不包含业务逻辑仅界面
    private void setMeasureResutViewVisibility(int state) {
        Log.i(TAG, "setMeasureResutViewVisibility: " + state);
        resultLayout.setVisibility(state);
        btReMeasure.setVisibility(state);
        btReMeasure.setText(R.string.remeasure);
        //通过intent启动该界面时,需要显示"提交"按钮
        btSubmit.setVisibility(View.GONE);
        btSubmit.setText(R.string.submit);
    }

    /**
     * 显示测量界面，包含相应的逻辑
     */
    private void showMeasureReadyViewAndInit() {
        setMeasureResutViewVisibility(View.INVISIBLE);
        setMeasureReadyViewVisibility(View.VISIBLE);
        measureBnt.setClickable(true);
    }

    /**
     * 显示测量结果界面，包含相应的逻辑
     */
    private void showMeasureResutView() {
        Log.i(TAG, "zhg showMeasureResutView: ");
        measureBnt.setClickable(false);
        float leng = callbackMeasureResult.getLength();
        float width = callbackMeasureResult.getWidth();
        float heigh = callbackMeasureResult.getHeight();
        retWidth.setText(width + "");
        retHeight.setText(heigh + "");
        retLength.setText(leng + "");

        String num = Util.getFormatNumberToString(leng * width * heigh / 1000000, 5);
        retVolume.setText(num);
        //清空测量过程超出边界的提示
        setMeasureReadyViewVisibility(View.INVISIBLE);
        setMeasureResutViewVisibility(View.VISIBLE);
    }

    /**
     * 显示测量结果界面，包含相应的逻辑
     */
    private void cleanMeasureResutView() {
        Log.i(TAG, "zhg cleanMeasureResutView: ");
        measureBnt.setClickable(true);
        /*float leng = callbackMeasureResult.getLength();
        float width = callbackMeasureResult.getWidth();
        float heigh = callbackMeasureResult.getHeight();
        retWidth.setText(width + "");
        retHeight.setText(heigh + "");
        retLength.setText(leng + "");
        DecimalFormat df = new DecimalFormat("0.00000");
        String num = df.format(volume);
        retVolume.setText(num);*/
        //清空测量过程超出边界的提示
        setMeasureReadyViewVisibility(View.VISIBLE);
        setMeasureResutViewVisibility(View.INVISIBLE);
    }

    /**
     * 显示测量结果界面，包含相应的逻辑
     */
/*    private void showBigobjectFirstSideMeasureResutView() {
        measureFinish = true;//暂停画面
        viewfinderView.setAimState(ViewfinderView.AIMED_STATE, "请继续测量第二面");
        measureBnt.setClickable(false);
        retLength.setText(bigLen1 + "");
        retWidth.setText(bigWid1 + "");
        retHeight.setText("---");
        retVolume.setText("---");
        //清空测量过程超出边界的提示
        toastText.setText("");
        setMeasureReadyViewVisibility(View.INVISIBLE);
        setMeasureResutViewVisibility(View.VISIBLE);
        //
        btReMeasure.setText("测另一面");
        btSubmit.setText("重测");
        btSubmit.setVisibility(View.VISIBLE);
    }*/
    /********************************/


    /**
     * 自动申请权限
     *
     * @param permissionList
     */
    private void autoObtainPermission(List<String> permissionList) {
        SimplePermissions.with(this)
                .constantRequest()
                .permission(permissionList)
                .request(new OnPermission() {
                    @Override
                    public void hasPermission(List<String> granted, boolean isAll) {
                        Log.i(TAG, "hasPermission: +isAll:" + isAll);
                    }

                    @Override
                    public void noPermission(List<String> denied, boolean quick) {
                        if (PermissionUtils.getManifestPermissions(MainActivity.this).containsAll(denied)) {
                            Toast.makeText(MainActivity.this, getString(R.string.grant_permission_manual), Toast.LENGTH_SHORT).show();
                            SimplePermissions.gotoPermissionSettings(MainActivity.this);
                        }
                        deniedList = denied;
                    }
                });
    }

    private void checkPermission() {
        int tarsdkversion = getApplicationInfo().targetSdkVersion;
        if (tarsdkversion >= 23) {
            autoObtainPermission(PermissionUtils.getManifestPermissions(this));
            if (SimplePermissions.isHasPermission(this, (String[]) PermissionUtils.getManifestPermissions(this).toArray())) {
                File file = new File(Environment.getExternalStorageDirectory().getAbsolutePath() + "/neolix/cubing");
                if (!file.exists()) {
                    file.mkdirs();
                }
            } else {
                if (deniedList != null) {
                    autoObtainPermission(deniedList);
                }
            }
        }
    }
    //物体轮廓 画笔
    private void initPaint() {
        for (int i = 0; i < 4; i++) {
            linePaints[i] = getSolidePaint(Color.WHITE, 5);
            pointPaints[i] = getSolidePaint(Color.WHITE, 5);
        }
    }

    public static Paint getSolidePaint(int color, int strokeWidth) {
        Paint paint = new Paint(Paint.ANTI_ALIAS_FLAG);
        paint.setAntiAlias(true);
        paint.setColor(color);//0x8000FF00 绿色
        paint.setStrokeWidth((float) strokeWidth);//2.5
        paint.setStyle(Paint.Style.FILL);
        return paint;
    }

    public void measure(View view) {
        int id = view.getId();
        switch (id) {
            case R.id.btn_measure:
                measure();
                break;
            case R.id.bt_remeasure:
                reMeasure();
                break;
        }
    }
}
